import { AuthListener, AuthOptions, ClientAdapter, Exclude, IAuth, Request, RequestInterceptor, Response, ResponseInterceptor, ServerAdapter, ServerConfiguration, Tokens, UsernamePasswordCredentials } from './auth-toolbox';
/**
 * Default entrypoint for auth-toolbox module.
 *
 * Load tokens from {@link TokenStorage}, support authentication methods like {@link login} and
 * {@link logout}, and automates the authentication of client provided with {@link ClientAdapter}.
 *
 * Options can be defined with {@link AuthOptions}.
 *
 * @param C Credentials type
 * @param R Client response type
 */
export declare class Auth<C = UsernamePasswordCredentials, R = any> implements IAuth<C, R>, RequestInterceptor, ResponseInterceptor {
    /**
     * @inheritDoc
     */
    usePersistentStorage: boolean;
    private serverAdapter;
    private serverConfiguration;
    private deferredServerConfiguration;
    private clientAdapter;
    private accessTokenDecoder?;
    private tokenStorage?;
    private persistentTokenStorage?;
    private tokenStorageAsync?;
    private persistentTokenStorageAsync?;
    private listeners;
    private excludes;
    private tokens?;
    private interceptors;
    private renewRunning;
    private renewPromises;
    /**
     * Builds an instance with given configuration.
     *
     * @param serverConfiguration Server configuration to use.
     *                            It may be a hardcoded configuration, or a configuration build from
     *                            a discovery service.
     * @param serverAdapter       Server adapter to use.
     * @param clientAdapter       Client adapter to user.
     * @param options             Options
     */
    constructor(serverConfiguration: ServerConfiguration | Promise<ServerConfiguration>, serverAdapter: ServerAdapter<C>, clientAdapter: ClientAdapter<R>, options?: AuthOptions | null);
    /**
     * @inheritDoc
     */
    get storageSync(): boolean;
    /**
     * @inheritDoc
     */
    loadTokensFromStorageAsync(): Promise<Tokens<C> | undefined>;
    /**
     * @inheritDoc
     */
    loadTokensFromStorage(): Tokens<C> | undefined;
    /**
     * @inheritDoc
     */
    release(): void;
    /**
     * @inheritDoc
     */
    addListener(...listeners: AuthListener[]): void;
    /**
     * @inheritDoc
     */
    removeListener(...listeners: AuthListener[]): void;
    addExclude(...excludes: Exclude[]): void;
    /**
     * @inheritDoc
     */
    getTokens(): Tokens<C> | undefined;
    /**
     * @inheritDoc
     */
    decodeAccessToken(): any | undefined;
    isExpiredAccessToken(offset?: number): boolean | undefined;
    /**
     * @inheritDoc
     */
    isAuthenticated(): boolean;
    getServerConfiguration(): Promise<ServerConfiguration>;
    /**
     * @inheritDoc
     */
    login(credentials: C): Promise<R>;
    private loginImpl;
    /**
     * @inheritDoc
     */
    renew(): Promise<R | void>;
    /**
     * @inheritDoc
     */
    logout(): Promise<R | void>;
    private initClientAdapter;
    private isLoginRequest;
    private isRenewRequest;
    private expired;
    private unsetTokensImplAsync;
    private setTokensImplAsync;
    private unsetTokensImpl;
    private setTokensImpl;
    /**
     * @inheritDoc
     */
    setTokens(tokens: Tokens<C> | undefined | null): void;
    /**
     * @inheritDoc
     */
    setTokensAsync(tokens: Tokens<C> | undefined | null): Promise<void>;
    private isExcluded;
    /**
     * @inheritDoc
     */
    interceptRequest(request: Request): Promise<boolean>;
    /**
     * @inheritDoc
     */
    interceptResponse(request: Request, response: Response): Promise<boolean>;
}
