var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import { DefaultTokenDecoder } from './token-decoder/default-token-decoder';
import { DefaultTokenStorage } from './token-storage/default-token-storage';
import { toTokenStorageAsync, toTokenStorageSync } from './token-storage/async-adapter';
var defaultAuthOptions = {
    accessTokenDecoder: new DefaultTokenDecoder(),
    tokenStorage: new DefaultTokenStorage(sessionStorage),
    persistentTokenStorage: new DefaultTokenStorage(localStorage),
    clientInterceptors: true
};
/**
 * Default entrypoint for auth-toolbox module.
 *
 * Load tokens from {@link TokenStorage}, support authentication methods like {@link login} and
 * {@link logout}, and automates the authentication of client provided with {@link ClientAdapter}.
 *
 * Options can be defined with {@link AuthOptions}.
 *
 * @param C Credentials type
 * @param R Client response type
 */
var Auth = /** @class */ (function () {
    /**
     * Builds an instance with given configuration.
     *
     * @param serverConfiguration Server configuration to use.
     *                            It may be a hardcoded configuration, or a configuration build from
     *                            a discovery service.
     * @param serverAdapter       Server adapter to use.
     * @param clientAdapter       Client adapter to user.
     * @param options             Options
     */
    function Auth(serverConfiguration, serverAdapter, clientAdapter, options) {
        var _this = this;
        /**
         * @inheritDoc
         */
        this.usePersistentStorage = false;
        this.listeners = [];
        this.excludes = [];
        this.interceptors = [];
        this.renewRunning = false;
        this.renewPromises = [];
        this.serverConfiguration = serverConfiguration;
        this.serverAdapter = serverAdapter;
        this.clientAdapter = clientAdapter;
        var effectiveOptions = Object.assign({}, defaultAuthOptions, options);
        this.accessTokenDecoder = effectiveOptions.accessTokenDecoder;
        this.tokenStorage = toTokenStorageSync(effectiveOptions.tokenStorage);
        this.tokenStorageAsync = toTokenStorageAsync(effectiveOptions.tokenStorage);
        this.persistentTokenStorage = toTokenStorageSync(effectiveOptions.persistentTokenStorage);
        this.persistentTokenStorageAsync = toTokenStorageAsync(effectiveOptions.persistentTokenStorage);
        if (effectiveOptions.listeners) {
            this.addListener.apply(this, effectiveOptions.listeners);
        }
        if (effectiveOptions.clientInterceptors) {
            this.initClientAdapter();
        }
        if (effectiveOptions.excludes) {
            this.addExclude.apply(this, effectiveOptions.excludes);
        }
        if (effectiveOptions.loadTokensFromStorage === undefined || effectiveOptions.loadTokensFromStorage === null) {
            // If undefined or null, tokens will be loaded only if storage supports sync loading.
            effectiveOptions.loadTokensFromStorage = this.storageSync;
        }
        if (effectiveOptions.loadTokensFromStorage) {
            if (this.storageSync) {
                this.loadTokensFromStorage();
                this.listeners.forEach(function (l) { return l.initialized && l.initialized(true); });
            }
            else {
                this.loadTokensFromStorageAsync()
                    .then(function () {
                    _this.listeners.forEach(function (l) { return l.initialized && l.initialized(true); });
                })
                    .catch(function (e) {
                    _this.listeners.forEach(function (l) { return l.initialized && l.initialized(false, e); });
                });
            }
        }
        else {
            this.listeners.forEach(function (l) { return l.initialized && l.initialized(false); });
        }
    }
    Object.defineProperty(Auth.prototype, "storageSync", {
        /**
         * @inheritDoc
         */
        get: function () {
            if (this.tokenStorageAsync && !this.tokenStorage) {
                return false;
            }
            if (this.persistentTokenStorageAsync && !this.persistentTokenStorage) {
                return false;
            }
            return true;
        },
        enumerable: false,
        configurable: true
    });
    /**
     * @inheritDoc
     */
    Auth.prototype.loadTokensFromStorageAsync = function () {
        return __awaiter(this, void 0, void 0, function () {
            var tokens, persistentTokens;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.tokenStorageAsync) return [3 /*break*/, 11];
                        return [4 /*yield*/, this.tokenStorageAsync.getTokens()];
                    case 1:
                        tokens = _a.sent();
                        if (!this.persistentTokenStorageAsync) return [3 /*break*/, 6];
                        return [4 /*yield*/, this.persistentTokenStorageAsync.getTokens()];
                    case 2:
                        persistentTokens = _a.sent();
                        if (persistentTokens) {
                            this.usePersistentStorage = true;
                        }
                        if (!tokens) {
                            tokens = persistentTokens;
                        }
                        if (!!tokens) return [3 /*break*/, 4];
                        return [4 /*yield*/, this.persistentTokenStorageAsync.clear()];
                    case 3:
                        _a.sent();
                        return [3 /*break*/, 6];
                    case 4: return [4 /*yield*/, this.persistentTokenStorageAsync.store(tokens)];
                    case 5:
                        _a.sent();
                        _a.label = 6;
                    case 6:
                        if (!!tokens) return [3 /*break*/, 8];
                        return [4 /*yield*/, this.unsetTokensImplAsync()];
                    case 7:
                        _a.sent();
                        return [3 /*break*/, 10];
                    case 8: return [4 /*yield*/, this.setTokensImplAsync(tokens)];
                    case 9:
                        _a.sent();
                        _a.label = 10;
                    case 10: return [2 /*return*/, tokens];
                    case 11: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * @inheritDoc
     */
    Auth.prototype.loadTokensFromStorage = function () {
        if (this.tokenStorageAsync) {
            if (!this.tokenStorage) {
                throw new Error('tokenStorage is async. Use loadTokensFromStorageAsync method instead');
            }
            var tokens = this.tokenStorage.getTokens();
            if (this.persistentTokenStorageAsync) {
                if (!this.persistentTokenStorage) {
                    throw new Error('persistentTokenStorage is async. Use loadTokensFromStorageAsync method instead');
                }
                var persistentTokens = this.persistentTokenStorage.getTokens();
                if (persistentTokens) {
                    this.usePersistentStorage = true;
                }
                if (!tokens) {
                    tokens = persistentTokens;
                }
                if (!tokens) {
                    this.persistentTokenStorage.clear();
                }
                else {
                    this.persistentTokenStorage.store(tokens);
                }
            }
            if (!tokens) {
                this.unsetTokensImpl();
            }
            else {
                this.setTokensImpl(tokens);
            }
            return tokens;
        }
    };
    /**
     * @inheritDoc
     */
    Auth.prototype.release = function () {
        for (var _i = 0, _a = this.interceptors; _i < _a.length; _i++) {
            var handle = _a[_i];
            handle();
        }
        this.removeListener.apply(this, this.listeners);
    };
    /**
     * @inheritDoc
     */
    Auth.prototype.addListener = function () {
        var _a;
        var listeners = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            listeners[_i] = arguments[_i];
        }
        (_a = this.listeners).push.apply(_a, listeners);
    };
    /**
     * @inheritDoc
     */
    Auth.prototype.removeListener = function () {
        var listeners = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            listeners[_i] = arguments[_i];
        }
        for (var _a = 0, listeners_1 = listeners; _a < listeners_1.length; _a++) {
            var listener = listeners_1[_a];
            var indexOf = this.listeners.indexOf(listener);
            if (indexOf > -1) {
                this.listeners.splice(indexOf, 1);
            }
        }
    };
    Auth.prototype.addExclude = function () {
        var excludes = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            excludes[_i] = arguments[_i];
        }
        var _loop_1 = function (exclude) {
            var excludeFunction = void 0;
            if (typeof exclude === 'string') {
                excludeFunction = function (request) { return request.url === exclude; };
            }
            else if (exclude instanceof RegExp) {
                excludeFunction = function (request) { return exclude.test(request.url); };
            }
            else {
                excludeFunction = exclude;
            }
            this_1.excludes.push(excludeFunction);
        };
        var this_1 = this;
        for (var _a = 0, excludes_1 = excludes; _a < excludes_1.length; _a++) {
            var exclude = excludes_1[_a];
            _loop_1(exclude);
        }
    };
    /**
     * @inheritDoc
     */
    Auth.prototype.getTokens = function () {
        return this.tokens;
    };
    /**
     * @inheritDoc
     */
    Auth.prototype.decodeAccessToken = function () {
        if (this.tokens) {
            if (!this.accessTokenDecoder || !this.accessTokenDecoder.decode) {
                throw new Error('An accessTokenDecoder supporting decode method should be defined to decode access token.');
            }
            return this.accessTokenDecoder.decode(this.tokens.access);
        }
    };
    Auth.prototype.isExpiredAccessToken = function (offset) {
        if (this.tokens) {
            if (!this.accessTokenDecoder || !this.accessTokenDecoder.isExpired) {
                throw new Error('An accessTokenDecoder supporting isExpired method should be defined to check if access token is expired.');
            }
            return this.accessTokenDecoder.isExpired(this.tokens.access, offset);
        }
    };
    /**
     * @inheritDoc
     */
    Auth.prototype.isAuthenticated = function () {
        return !!this.tokens;
    };
    Auth.prototype.getServerConfiguration = function () {
        return __awaiter(this, void 0, void 0, function () {
            var _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        if (!!this.deferredServerConfiguration) return [3 /*break*/, 2];
                        _a = this;
                        return [4 /*yield*/, this.serverConfiguration];
                    case 1:
                        _a.deferredServerConfiguration = _b.sent();
                        _b.label = 2;
                    case 2: return [2 /*return*/, this.deferredServerConfiguration];
                }
            });
        });
    };
    /**
     * @inheritDoc
     */
    Auth.prototype.login = function (credentials) {
        return __awaiter(this, void 0, void 0, function () {
            var response;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.loginImpl(credentials)];
                    case 1:
                        response = _a.sent();
                        this.listeners.forEach(function (l) { return l.login && l.login(); });
                        return [2 /*return*/, response];
                }
            });
        });
    };
    Auth.prototype.loginImpl = function (credentials) {
        return __awaiter(this, void 0, void 0, function () {
            var serverConfiguration, request, response, tokens;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getServerConfiguration()];
                    case 1:
                        serverConfiguration = _a.sent();
                        request = this.serverAdapter.asLoginRequest(serverConfiguration.loginEndpoint, credentials);
                        return [4 /*yield*/, this.clientAdapter.login(request)];
                    case 2:
                        response = _a.sent();
                        tokens = this.serverAdapter.getResponseTokens(response);
                        if (this.usePersistentStorage && !serverConfiguration.renewEndpoint) {
                            tokens.credentials = credentials;
                        }
                        return [4 /*yield*/, this.setTokensImplAsync(tokens)];
                    case 3:
                        _a.sent();
                        return [2 /*return*/, response];
                }
            });
        });
    };
    /**
     * @inheritDoc
     */
    Auth.prototype.renew = function () {
        return __awaiter(this, void 0, void 0, function () {
            var serverConfiguration, response, request, tokens, _i, _a, renewTokenPromise, err_1, _b, _c, renewTokenPromise, renewTokenPromise_1;
            var _this = this;
            return __generator(this, function (_d) {
                switch (_d.label) {
                    case 0: return [4 /*yield*/, this.getServerConfiguration()];
                    case 1:
                        serverConfiguration = _d.sent();
                        if (!(this.tokens && serverConfiguration)) return [3 /*break*/, 12];
                        if (!!this.renewRunning) return [3 /*break*/, 11];
                        _d.label = 2;
                    case 2:
                        _d.trys.push([2, 8, 9, 10]);
                        this.renewRunning = true;
                        response = void 0;
                        if (!serverConfiguration.renewEndpoint) return [3 /*break*/, 4];
                        if (!this.tokens.refresh) {
                            throw new Error('No refresh token available to renew');
                        }
                        request = this.serverAdapter.asRenewRequest(serverConfiguration.renewEndpoint, this.tokens.refresh);
                        return [4 /*yield*/, this.clientAdapter.renew(request)];
                    case 3:
                        response = _d.sent();
                        return [3 /*break*/, 6];
                    case 4:
                        if (!this.tokens.credentials) {
                            throw new Error('Credentials are not available. ' +
                                'usePersistentStorage should be true to allow renew method without renewEndpoint and refresh token.');
                        }
                        return [4 /*yield*/, this.loginImpl(this.tokens.credentials)];
                    case 5:
                        response = _d.sent();
                        _d.label = 6;
                    case 6:
                        tokens = this.serverAdapter.getResponseTokens(response);
                        return [4 /*yield*/, this.setTokensImplAsync(tokens)];
                    case 7:
                        _d.sent();
                        for (_i = 0, _a = this.renewPromises; _i < _a.length; _i++) {
                            renewTokenPromise = _a[_i];
                            renewTokenPromise.resolve(response);
                        }
                        this.listeners.forEach(function (l) { return l.renew && l.renew(); });
                        return [2 /*return*/, response];
                    case 8:
                        err_1 = _d.sent();
                        for (_b = 0, _c = this.renewPromises; _b < _c.length; _b++) {
                            renewTokenPromise = _c[_b];
                            renewTokenPromise.reject(err_1);
                        }
                        throw err_1;
                    case 9:
                        this.renewRunning = false;
                        this.renewPromises = [];
                        return [7 /*endfinally*/];
                    case 10: return [3 /*break*/, 12];
                    case 11:
                        renewTokenPromise_1 = new Promise(function (resolve, reject) {
                            _this.renewPromises.push({ promise: renewTokenPromise_1, resolve: resolve, reject: reject });
                        });
                        return [2 /*return*/, renewTokenPromise_1];
                    case 12: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * @inheritDoc
     */
    Auth.prototype.logout = function () {
        return __awaiter(this, void 0, void 0, function () {
            var response, serverConfiguration, request;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.tokens) return [3 /*break*/, 5];
                        response = void 0;
                        return [4 /*yield*/, this.getServerConfiguration()];
                    case 1:
                        serverConfiguration = _a.sent();
                        if (!(serverConfiguration.logoutEndpoint && this.tokens.refresh)) return [3 /*break*/, 3];
                        request = this.serverAdapter.asLogoutRequest(serverConfiguration.logoutEndpoint, this.tokens.refresh);
                        return [4 /*yield*/, this.clientAdapter.logout(request)];
                    case 2:
                        response = _a.sent();
                        _a.label = 3;
                    case 3: return [4 /*yield*/, this.unsetTokensImplAsync()];
                    case 4:
                        _a.sent();
                        this.listeners.forEach(function (l) { return l.logout && l.logout(); });
                        return [2 /*return*/, response];
                    case 5: return [2 /*return*/];
                }
            });
        });
    };
    Auth.prototype.initClientAdapter = function () {
        this.interceptors.push(this.clientAdapter.setupRequestInterceptor(this));
        this.interceptors.push(this.clientAdapter.setupErrorResponseInterceptor(this));
    };
    Auth.prototype.isLoginRequest = function (request) {
        return __awaiter(this, void 0, void 0, function () {
            var serverConfiguration;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getServerConfiguration()];
                    case 1:
                        serverConfiguration = _a.sent();
                        if (serverConfiguration.loginEndpoint &&
                            serverConfiguration.loginEndpoint.method.toLowerCase() === request.method.toLowerCase() &&
                            serverConfiguration.loginEndpoint.url === request.url) {
                            return [2 /*return*/, true];
                        }
                        return [2 /*return*/, false];
                }
            });
        });
    };
    Auth.prototype.isRenewRequest = function (request) {
        return __awaiter(this, void 0, void 0, function () {
            var serverConfiguration;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getServerConfiguration()];
                    case 1:
                        serverConfiguration = _a.sent();
                        if (serverConfiguration.renewEndpoint &&
                            serverConfiguration.renewEndpoint.method.toLowerCase() === request.method.toLowerCase() &&
                            serverConfiguration.renewEndpoint.url === request.url) {
                            return [2 /*return*/, true];
                        }
                        return [2 /*return*/, false];
                }
            });
        });
    };
    Auth.prototype.expired = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.tokens) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.unsetTokensImplAsync()];
                    case 1:
                        _a.sent();
                        this.listeners.forEach(function (l) { return l.expired && l.expired(); });
                        _a.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        });
    };
    Auth.prototype.unsetTokensImplAsync = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.tokens = undefined;
                        if (!this.persistentTokenStorageAsync) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.persistentTokenStorageAsync.clear()];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2:
                        if (!this.tokenStorageAsync) return [3 /*break*/, 4];
                        return [4 /*yield*/, this.tokenStorageAsync.clear()];
                    case 3:
                        _a.sent();
                        _a.label = 4;
                    case 4:
                        this.listeners.forEach(function (l) { return l.tokensChanged && l.tokensChanged(); });
                        return [2 /*return*/];
                }
            });
        });
    };
    Auth.prototype.setTokensImplAsync = function (tokens) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.tokenStorageAsync) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.tokenStorageAsync.store(tokens)];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2:
                        if (!this.persistentTokenStorageAsync) return [3 /*break*/, 6];
                        if (!this.usePersistentStorage) return [3 /*break*/, 4];
                        return [4 /*yield*/, this.persistentTokenStorageAsync.store(tokens)];
                    case 3:
                        _a.sent();
                        return [3 /*break*/, 6];
                    case 4: return [4 /*yield*/, this.persistentTokenStorageAsync.clear()];
                    case 5:
                        _a.sent();
                        _a.label = 6;
                    case 6:
                        this.tokens = tokens;
                        this.listeners.forEach(function (l) { return l.tokensChanged && l.tokensChanged(_this.tokens); });
                        return [2 /*return*/];
                }
            });
        });
    };
    Auth.prototype.unsetTokensImpl = function () {
        if (this.tokenStorageAsync) {
            if (!this.tokenStorage) {
                throw new Error('tokenStorage is async. Use setTokensAsync method instead');
            }
            this.tokenStorage.clear();
        }
        if (this.persistentTokenStorageAsync) {
            if (!this.persistentTokenStorage) {
                throw new Error('persistentTokenStorage is async. Use setTokensAsync method instead');
            }
            this.persistentTokenStorage.clear();
        }
        this.tokens = undefined;
        this.listeners.forEach(function (l) { return l.tokensChanged && l.tokensChanged(); });
    };
    Auth.prototype.setTokensImpl = function (tokens) {
        var _this = this;
        if (this.tokenStorageAsync) {
            if (!this.tokenStorage) {
                throw new Error('tokenStorage is async. Use setTokensAsync method instead');
            }
            this.tokenStorage.store(tokens);
        }
        if (this.persistentTokenStorageAsync) {
            if (!this.persistentTokenStorage) {
                throw new Error('persistentTokenStorage is async. Use setTokensAsync method instead');
            }
            if (this.usePersistentStorage) {
                this.persistentTokenStorage.store(tokens);
            }
            else {
                this.persistentTokenStorage.clear();
            }
        }
        this.tokens = tokens;
        this.listeners.forEach(function (l) { return l.tokensChanged && l.tokensChanged(_this.tokens); });
    };
    /**
     * @inheritDoc
     */
    Auth.prototype.setTokens = function (tokens) {
        if (tokens) {
            this.setTokensImpl(tokens);
        }
        else {
            this.unsetTokensImpl();
        }
    };
    /**
     * @inheritDoc
     */
    Auth.prototype.setTokensAsync = function (tokens) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!tokens) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.setTokensImplAsync(tokens)];
                    case 1:
                        _a.sent();
                        return [3 /*break*/, 4];
                    case 2: return [4 /*yield*/, this.unsetTokensImplAsync()];
                    case 3:
                        _a.sent();
                        _a.label = 4;
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    Auth.prototype.isExcluded = function (request, response) {
        for (var _i = 0, _a = this.excludes; _i < _a.length; _i++) {
            var exclude = _a[_i];
            if (exclude(request, response)) {
                return true;
            }
        }
        return false;
    };
    /**
     * @inheritDoc
     */
    Auth.prototype.interceptRequest = function (request) {
        return __awaiter(this, void 0, void 0, function () {
            var tokens, isLoginRequest, isRenewRequest, err_2;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.isExcluded(request)) {
                            return [2 /*return*/, false];
                        }
                        tokens = this.getTokens();
                        return [4 /*yield*/, this.isLoginRequest(request)];
                    case 1:
                        isLoginRequest = _a.sent();
                        if (!(tokens && tokens.access && !isLoginRequest)) return [3 /*break*/, 11];
                        if (!tokens.refresh) return [3 /*break*/, 9];
                        return [4 /*yield*/, this.isRenewRequest(request)];
                    case 2:
                        isRenewRequest = _a.sent();
                        if (!(!isRenewRequest &&
                            this.accessTokenDecoder &&
                            this.accessTokenDecoder.isExpired &&
                            this.accessTokenDecoder.isExpired(tokens.access))) return [3 /*break*/, 8];
                        _a.label = 3;
                    case 3:
                        _a.trys.push([3, 5, , 7]);
                        return [4 /*yield*/, this.renew()];
                    case 4:
                        _a.sent();
                        return [3 /*break*/, 7];
                    case 5:
                        err_2 = _a.sent();
                        return [4 /*yield*/, this.expired()];
                    case 6:
                        _a.sent();
                        throw err_2;
                    case 7:
                        tokens = this.getTokens();
                        _a.label = 8;
                    case 8:
                        this.serverAdapter.setAccessToken(request, tokens.access.value);
                        return [2 /*return*/, true];
                    case 9:
                        this.serverAdapter.setAccessToken(request, tokens.access.value);
                        if (!(this.accessTokenDecoder &&
                            this.accessTokenDecoder.isExpired &&
                            this.accessTokenDecoder.isExpired(tokens.access))) return [3 /*break*/, 11];
                        return [4 /*yield*/, this.expired()];
                    case 10:
                        _a.sent();
                        throw new Error('Access token is expired');
                    case 11: return [2 /*return*/, false];
                }
            });
        });
    };
    /**
     * @inheritDoc
     */
    Auth.prototype.interceptResponse = function (request, response) {
        return __awaiter(this, void 0, void 0, function () {
            var tokens, refreshToken, isRenewRequest, err_3;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.isExcluded(request, response)) {
                            return [2 /*return*/, false];
                        }
                        tokens = this.getTokens();
                        refreshToken = tokens && tokens.refresh ? tokens.refresh.value : undefined;
                        if (!refreshToken) return [3 /*break*/, 10];
                        return [4 /*yield*/, this.isRenewRequest(request)];
                    case 1:
                        isRenewRequest = _a.sent();
                        if (!(!isRenewRequest && this.serverAdapter.accessTokenHasExpired(request, response))) return [3 /*break*/, 7];
                        _a.label = 2;
                    case 2:
                        _a.trys.push([2, 4, , 6]);
                        return [4 /*yield*/, this.renew()];
                    case 3:
                        _a.sent();
                        return [2 /*return*/, true];
                    case 4:
                        err_3 = _a.sent();
                        return [4 /*yield*/, this.expired()];
                    case 5:
                        _a.sent();
                        return [2 /*return*/, false];
                    case 6: return [3 /*break*/, 9];
                    case 7:
                        if (!(isRenewRequest && this.serverAdapter.refreshTokenHasExpired(request, response))) return [3 /*break*/, 9];
                        return [4 /*yield*/, this.expired()];
                    case 8:
                        _a.sent();
                        return [2 /*return*/, false];
                    case 9: return [2 /*return*/, false];
                    case 10:
                        if (!this.serverAdapter.accessTokenHasExpired(request, response)) return [3 /*break*/, 12];
                        return [4 /*yield*/, this.expired()];
                    case 11:
                        _a.sent();
                        _a.label = 12;
                    case 12: return [2 /*return*/, false];
                }
            });
        });
    };
    return Auth;
}());
export { Auth };
//# sourceMappingURL=auth.js.map