import { ClientAdapter, Request, Response, ServerAdapter, ServerConfiguration, ServerEndpoint, Token, Tokens, UsernamePasswordCredentials } from '../auth-toolbox';
export interface LoginResponse {
    access_token: string;
    expires_in: number;
    refresh_expires_in: number;
    refresh_token: string;
}
export interface OpenidConfiguration {
    token_endpoint: string;
    userinfo_endpoint: string;
    end_session_endpoint: string;
}
/**
 * Provides a {@link ServerConfiguration} through [OpenID discovery](https://openid.net/specs/openid-connect-discovery-1_0.html).
 *
 * It requests `{issuerUrl}/.well-known/openid-configuration` to retrieve login, renew and logout
 * endpoints.
 *
 * It should be given to {@link Auth} constructor.
 *
 * @param client client adapter to request discovery endpoint with.
 * @param issuerUrl url of the authentication realm (without `/.well-known/openid-configuration`).
 */
export declare function openidConnectDiscovery<R>(client: ClientAdapter<R>, issuerUrl: string): Promise<ServerConfiguration>;
/**
 * Implements OpenID Connect "Resource Owner Password Credentials Grant", it should be given to
 * {@link Auth} constructor.
 */
export declare class OpenidConnectAdapter implements ServerAdapter {
    asLoginRequest(loginEndpoint: ServerEndpoint, credentials: UsernamePasswordCredentials): Request;
    asLogoutRequest(logoutEndpoint: ServerEndpoint, refreshToken: Token): Request;
    asRenewRequest(renewEndpoint: ServerEndpoint, refreshToken: Token): Request;
    getResponseTokens(response: Response): Tokens<UsernamePasswordCredentials>;
    setAccessToken(request: Request, accessToken: string): void;
    accessTokenHasExpired(request: Request, response: Response): boolean;
    refreshTokenHasExpired(request: Request, response: Response): boolean;
}
