var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import { stringify } from 'querystring';
/**
 * Provides a {@link ServerConfiguration} through [OpenID discovery](https://openid.net/specs/openid-connect-discovery-1_0.html).
 *
 * It requests `{issuerUrl}/.well-known/openid-configuration` to retrieve login, renew and logout
 * endpoints.
 *
 * It should be given to {@link Auth} constructor.
 *
 * @param client client adapter to request discovery endpoint with.
 * @param issuerUrl url of the authentication realm (without `/.well-known/openid-configuration`).
 */
export function openidConnectDiscovery(client, issuerUrl) {
    return __awaiter(this, void 0, void 0, function () {
        var clientResponse, response, openidConfiguration;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, client.request({
                        method: 'GET',
                        url: issuerUrl + '/.well-known/openid-configuration'
                    })];
                case 1:
                    clientResponse = _a.sent();
                    response = client.asResponse(clientResponse);
                    openidConfiguration = response.data;
                    return [2 /*return*/, {
                            loginEndpoint: { method: 'POST', url: openidConfiguration.token_endpoint },
                            renewEndpoint: { method: 'POST', url: openidConfiguration.token_endpoint },
                            logoutEndpoint: { method: 'POST', url: openidConfiguration.end_session_endpoint }
                        }];
            }
        });
    });
}
/**
 * Implements OpenID Connect "Resource Owner Password Credentials Grant", it should be given to
 * {@link Auth} constructor.
 */
var OpenidConnectAdapter = /** @class */ (function () {
    function OpenidConnectAdapter() {
    }
    OpenidConnectAdapter.prototype.asLoginRequest = function (loginEndpoint, credentials) {
        var rawData = {
            grant_type: 'password',
            username: credentials.username,
            password: credentials.password
        };
        var data = stringify(rawData);
        var headers = { 'Content-Type': 'application/x-www-form-urlencoded' };
        return __assign(__assign({}, loginEndpoint), { data: data, headers: headers });
    };
    OpenidConnectAdapter.prototype.asLogoutRequest = function (logoutEndpoint, refreshToken) {
        var rawData = {
            refresh_token: refreshToken.value
        };
        var data = stringify(rawData);
        var headers = { 'Content-Type': 'application/x-www-form-urlencoded' };
        return __assign(__assign({}, logoutEndpoint), { data: data, headers: headers });
    };
    OpenidConnectAdapter.prototype.asRenewRequest = function (renewEndpoint, refreshToken) {
        var rawData = {
            grant_type: 'refresh_token',
            refresh_token: refreshToken.value
        };
        var data = stringify(rawData);
        var headers = { 'Content-Type': 'application/x-www-form-urlencoded' };
        return __assign(__assign({}, renewEndpoint), { data: data, headers: headers });
    };
    OpenidConnectAdapter.prototype.getResponseTokens = function (response) {
        var loginResponse = response.data;
        if (!loginResponse.access_token) {
            throw new Error('No access token found in response');
        }
        var tokens = {
            access: { value: loginResponse.access_token }
        };
        if (loginResponse.expires_in) {
            var expiresAt = new Date();
            expiresAt.setSeconds(expiresAt.getSeconds() + loginResponse.expires_in);
            tokens.access.expiresAt = expiresAt;
        }
        if (loginResponse.refresh_token) {
            tokens.refresh = { value: loginResponse.refresh_token };
            if (loginResponse.refresh_expires_in) {
                var expiresAt = new Date();
                expiresAt.setSeconds(expiresAt.getSeconds() + loginResponse.refresh_expires_in);
                tokens.refresh.expiresAt = expiresAt;
            }
        }
        return tokens;
    };
    OpenidConnectAdapter.prototype.setAccessToken = function (request, accessToken) {
        if (!request.headers) {
            request.headers = {};
        }
        request.headers.Authorization = 'Bearer ' + accessToken;
    };
    OpenidConnectAdapter.prototype.accessTokenHasExpired = function (request, response) {
        return (!!request.headers &&
            !!request.headers.Authorization &&
            response.status === 401 &&
            response.data.error === 'invalid_token');
    };
    OpenidConnectAdapter.prototype.refreshTokenHasExpired = function (request, response) {
        return (!!request.headers &&
            !!request.headers.Authorization &&
            response.status === 400 &&
            response.data.error === 'invalid_grant');
    };
    return OpenidConnectAdapter;
}());
export { OpenidConnectAdapter };
//# sourceMappingURL=openid-connect-adapter.js.map