/**
 * Default implementation of {@link TokenDecoder}. It doesn't support decoding {@link Token} and checks the
 * expiration date from the server response.
 *
 * It's the default value on {@link Auth} constructor for {@link AuthOptions.accessTokenDecoder}.
 */
var DefaultTokenDecoder = /** @class */ (function () {
    function DefaultTokenDecoder(offsetSeconds) {
        if (offsetSeconds === void 0) { offsetSeconds = 0; }
        this.offset = offsetSeconds * 1000;
    }
    DefaultTokenDecoder.prototype.isExpired = function (token) {
        if (token.expiresAt) {
            var now = new Date().getTime() - this.offset;
            if (now >= token.expiresAt.getTime()) {
                return true;
            }
        }
        return false;
    };
    return DefaultTokenDecoder;
}());
export { DefaultTokenDecoder };
//# sourceMappingURL=default-token-decoder.js.map