var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { DefaultTokenDecoder } from './default-token-decoder';
import { decode } from 'jsonwebtoken';
/**
 * {@link TokenDecoder} based on jsonwebtoken package to be used with JWT {@link Token}.
 * It check expiration dates from both server response and JWT `exp` claim, and supports decoding
 * of the JWT token.
 *
 * It should be given to {@link Auth} constructor through {@link AuthOptions.accessTokenDecoder}.
 */
var JwtTokenDecoder = /** @class */ (function (_super) {
    __extends(JwtTokenDecoder, _super);
    /**
     * @param expiredOffset Offset in millisecond to consider the token as expired.
     *                      use NaN to disable expiration from token decoding
     */
    function JwtTokenDecoder(expiredOffset) {
        if (expiredOffset === void 0) { expiredOffset = 0; }
        return _super.call(this, expiredOffset) || this;
    }
    JwtTokenDecoder.prototype.decode = function (token) {
        return decode(token.value);
    };
    JwtTokenDecoder.prototype.isExpired = function (token, offset) {
        if (_super.prototype.isExpired.call(this, token)) {
            return true;
        }
        if (offset === undefined) {
            offset = this.offset;
        }
        if (isNaN(offset)) {
            return false;
        }
        var now = Math.round((new Date().getTime() - offset) / 1000);
        var decoded = this.decode(token);
        if ('exp' in decoded && now >= decoded.exp) {
            return true;
        }
        return false;
    };
    return JwtTokenDecoder;
}(DefaultTokenDecoder));
export { JwtTokenDecoder };
//# sourceMappingURL=jwt-token-decoder.js.map